<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Block;

use Flat101\EnhancedEcommerce\Api\Data\DataLayerInterface;
//use Flat101\EnhancedEcommerce\DataProvider\CurrentProduct;
use Flat101\Base\Service\GetCurrentProductService As CurrentProduct;
use Magento\Framework\View\Element\Template;
use Flat101\EnhancedEcommerce\Api\DataLayerListInterface;
use Flat101\EnhancedEcommerce\Factory\DataLayerFactory;
use Flat101\EnhancedEcommerce\Helper\Config;
use Magento\Framework\App\Response\RedirectInterface;
use Magento\Framework\Event\Manager;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Framework\UrlInterface;

class DataLayer extends AbstractDatalayer
{
    /** @var CurrentProduct */
    protected $currentProduct;

    public function __construct(
        CurrentProduct $currentProduct,
        Template\Context $context,
        DataLayerListInterface $dataLayerList,
        DataLayerFactory $dataLayerFactory,
        SerializerInterface $serializer,
        UrlInterface $url,
        Config $config,
        RedirectInterface $redirect,
        Manager $eventManage,
        array $data = []
    ) {
        parent::__construct($context, $dataLayerList, $dataLayerFactory, $serializer, $url, $config, $redirect, $eventManage, $data);
        $this->currentProduct = $currentProduct;
    }
    public function getJsonDataLayers(): ?string
    {
        $data = [];

        if (null === $this->getTypeDataLayers()) {
            return null;
        }

        foreach ($this->getTypeDataLayers() as $typeDataLayer) {
            $instance = $this->getInstance($typeDataLayer);

            if (!($instance instanceof DataLayerInterface)) {
                continue;
            }

            /** @var DataLayerInterface $layer */
            $layer = $instance->getLayer();

            if (null !== $layer) {
                if (\is_array($layer)) {
                    foreach ($layer as $item) {
                        $data[] = $item;
                    }
                } else {
                    $data[] = $layer;
                }

                $this->codes[] = $instance->getCode();
            }
        }

        $this->eventManage->dispatch('flat101_enhanced_ecommerce_after_build_layer', ['dataLayer' => $data]);

        return $this->serializer->serialize($data);
    }

    public function getAjaxUrl(): string
    {
        return $this->getUrl('flat101_enhancedecommerce/impression/view');
    }

    public function getCurrentUrl(): string
    {
        return $this->url->getCurrentUrl();
    }

    public function getRefererUrl(): string
    {
        return $this->redirect->getRefererUrl();
    }

    public function getCurrentProductId(): ?int
    {
        $product = $this->currentProduct->getProduct();

        return null === $product ? null : (int) $product->getId();
    }
}
