<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayer\Impressions;

use Flat101\EnhancedEcommerce\Api\Data\DataLayerInterface;
use Flat101\EnhancedEcommerce\Model\DataLayer\Dto;
use Flat101\EnhancedEcommerce\Model\DataLayer\Generator\Impression;
use Flat101\EnhancedEcommerce\Model\Handler\ProductHandler;
use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\Product as ProductEntity;
use Magento\Catalog\Model\Product\Visibility as ProductVisibility;
use Magento\Store\Model\StoreManagerInterface;

class ProductRelatedView implements DataLayerInterface
{
    use UnsetProperty;

    public const CODE = 'product-related-view';

    private const RELATED = 'CrossSelling - Related Products';

    /** @var StoreManagerInterface */
    protected $storeManager;

    /** @var Impression */
    protected $impressionGenerator;

    /** @var Product */
    protected $productHandler;

    /** @var ProductRepository */
    protected $productRepository;

    /** @var ProductVisibility */
    protected $catalogProductVisibility;

    public function __construct(
        Impression $impressionGenerator,
        StoreManagerInterface $storeManager,
        ProductHandler $productHandler,
        ProductRepositoryInterface $productRepository,
        ProductVisibility $catalogProductVisibility
    ) {
        $this->storeManager = $storeManager;
        $this->impressionGenerator = $impressionGenerator;
        $this->productHandler = $productHandler;
        $this->productRepository = $productRepository;
        $this->catalogProductVisibility = $catalogProductVisibility;
    }

    public function getCode(): string
    {
        return static::CODE;
    }

    /**
     * {@inheritdoc}
     */
    public function getLayer(): ?Dto\Ecommerce
    {
        /** @var ProductEntity $product */
        $product = $this->productHandler->getProduct();

        if (is_null($product)) {
            return null;
        }

        // Get related Products with sort order
        $relatedProducts = $product->getRelatedProductCollection()->addAttributeToSelect(
            'required_options'
        )->setPositionOrder()->addStoreFilter();
        // Set visibility
        $relatedProducts->setVisibility($this->catalogProductVisibility->getVisibleInCatalogIds());

        if (0 === \count($relatedProducts)) {
            return null;
        }

        $impressionProducts = [];
        $position = 0;
        /** @var Product $relatedProduct */
        foreach ($relatedProducts as $relatedProduct) {
            $entity = $this->productRepository->getById($relatedProduct->getId());
            $position++;

            $impressionProducts[] = $this->impressionGenerator->generate($entity, self::RELATED, ['position' => $position]);
        }

        $impression = new Dto\Impression\Impression();
        $impression->currencyCode = $this->storeManager->getStore()->getCurrentCurrency()->getCode();
        $impression->impressions = $impressionProducts;
        $this->unset($impression, ['category']);

        $ecommerce = new Dto\Ecommerce();
        $ecommerce->event = 'productList';
        $ecommerce->ecommerce = $impression;

        $this->unset($ecommerce, ['eventCategory','eventAction','eventLabel']);

        return $ecommerce;
    }
}
