<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayer;

use Flat101\EnhancedEcommerce\Api\Data\DataLayerInterface;
use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Flat101\EnhancedEcommerce\Helper\Config;

class ProductView implements DataLayerInterface
{
    use UnsetProperty;

    public const CODE = 'product-view';

    private const EXCLUDE_FIELDS = [
        'quantity',
    ];

    /** @var Generator\Product */
    protected $productGenerator;

    /** @var Config */
    protected $configHelper;

    public function __construct(
        Generator\Product $productGenerator,
        Config $configHelper
    ) {
        $this->productGenerator = $productGenerator;
        $this->configHelper = $configHelper;
    }

    public function getCode(): string
    {
        return static::CODE;
    }

    public function getLayer(): ?Dto\Ecommerce
    {
        $product = $this->productGenerator->generate(null);

        if (is_null($product)) {
            return null;
        }

        $this->unset($product, self::EXCLUDE_FIELDS);

        $actionField = new Dto\Impression\ActionField();
        $catNames = explode($this->configHelper->getCategorySeparator(), $product->category);
        $actionField->list = trim(array_pop($catNames));

        $detailEcommerceDto = new Dto\EcommerceDetails();
        $detailEcommerceDto->actionField = $actionField;
        $detailEcommerceDto->products[] = $product;

        $detailsDto = new Dto\Details();
        $detailsDto->detail = $detailEcommerceDto;

        $ecommerce = new Dto\Ecommerce();
        $ecommerce->event = 'productDetail';
        $ecommerce->ecommerce = $detailsDto;

        $this->unset($ecommerce, ['eventCategory','eventAction','eventLabel']);

        return $ecommerce;
    }
}
