<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\Handler;

use Flat101\EnhancedEcommerce\Model\DataLayer\Generator\Impression;
use Flat101\Base\Service\GetCurrentCategoryService As CategoryRegistry;
use Flat101\EnhancedEcommerce\Model\Session;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Catalog\Block\Product\ProductList\Toolbar;

class CatalogImpressionHandler implements ImpressionHandlerInterface
{
    private const CATALOG_LIST = 'Catalog List';

    /** @var Session */
    private $session;

    /** @var Impression */
    private $impressionGenerator;

    /** @var Toolbar */
    protected $toolbar;

    /** @var CategoryRegistry */
    protected $categoryRegistry;

    public function __construct(
        Session $session,
        Impression $impressionGenerator,
        Toolbar $toolbar,
        CategoryRegistry $categoryRegistry
    ) {
        $this->session = $session;
        $this->impressionGenerator = $impressionGenerator;
        $this->toolbar = $toolbar;
        $this->categoryRegistry = $categoryRegistry;
    }

    public function handle(Collection $collection): void
    {
        $this->session->getImpressionCatalogProducts(true);

        $listName = $this->categoryRegistry->getCategory()->getName() ?? self::CATALOG_LIST;
        $pageSize = $this->toolbar->getLimit();
        $pageNumber = $this->toolbar->getCurrentPage();
        $position = ($pageNumber - 1) * $pageSize;

        /** @var \Magento\Catalog\Model\Product $product */
        foreach ($collection as $product) {
            $position++;
            $this->session->setImpressionCatalogProducts(
                $this->impressionGenerator->generate($product, $listName, ['position' => $position])
            );
        }
    }
}
