<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\Handler;

use Flat101\EnhancedEcommerce\Model\DataLayer\Generator\Impression;
use Flat101\EnhancedEcommerce\Model\Session;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Catalog\Block\Product\ProductList\Toolbar;

class SearchImpressionHandler implements ImpressionHandlerInterface
{
    private const SEARCH_RESULT = 'Search Results';

    /** @var Session */
    private $session;

    /** @var Impression */
    private $impressionGenerator;

    /** @var Toolbar */
    protected $toolbar;

    public function __construct(
        Session $session,
        Impression $impressionGenerator,
        Toolbar $toolbar
    ) {
        $this->session = $session;
        $this->impressionGenerator = $impressionGenerator;
        $this->toolbar = $toolbar;
    }

    public function handle(Collection $collection): void
    {
        $this->session->getImpressionSearchProducts(true);

        $pageSize = $this->toolbar->getLimit();
        $pageNumber = $this->toolbar->getCurrentPage();
        $position = ($pageNumber - 1) * $pageSize;

        /** @var \Magento\Catalog\Model\Product $product */
        foreach ($collection as $product) {
            $position++;
            $this->session->setImpressionSearchProducts(
                $this->impressionGenerator->generate($product, self::SEARCH_RESULT, ['position' => $position])
            );
        }
    }
}
