<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model;

use Flat101\EnhancedEcommerce\Model\DataLayer\Dto;
use Magento\Framework\Session\SessionManager;

class Session extends SessionManager
{
    const KEY_ADD_PRODUCTS_TO_CART = 'add_products_to_cart';
    const KEY_REMOVED_PRODUCTS_FROM_CART = 'removed_products_from_cart';
    const KEY_CHECKOUT_STEPS = 'checkout_step';
    const KEY_IMPRESSION_CATALOG_PRODUCTS = 'impression_catalog_products';
    const KEY_IMPRESSION_SEARCH_PRODUCTS = 'impression_search_products';
    const KEY_CLICK_IMPRESSION_PRODUCTS = 'click_impression_products';

    const KEY_PRODUCT_ADDITIONAL_INFO = 'product_additional_info';
    const KEY_BEHAVIORAL_EVENT = 'behavioral_events';

    public function setAddProductToCart($product): Session
    {
        $items = $this->getAddProductToCart();

        $this->storage->setData(self::KEY_ADD_PRODUCTS_TO_CART, array_merge($items, [$product]));

        return $this;
    }

    public function getAddProductToCart(bool $clear = true): array
    {
        $products = $this->getData(self::KEY_ADD_PRODUCTS_TO_CART, $clear);

        if (null === $products) {
            return [];
        }

        if (!is_array($products)) {
            return  [];
        }

        return 0 < count($products) ? $products : [];
    }

    public function setRemovedProductFromCart($product): Session
    {
        $items = $this->getRemovedProductFromCart();

        $this->storage->setData(self::KEY_REMOVED_PRODUCTS_FROM_CART, array_merge($items, [$product]));

        return $this;
    }

    public function getRemovedProductFromCart(bool $clear = false): array
    {
        $products = $this->getData(self::KEY_REMOVED_PRODUCTS_FROM_CART, $clear);

        if (null === $products) {
            return [];
        }

        if (!is_array($products)) {
            return  [];
        }

        return 0 < count($products) ? $products : [];
    }

    public function setCheckoutStep(Dto\Ecommerce $paymentStepCheckout): Session
    {
        $steps = $this->getCheckoutSteps();

        $this->storage->setData(self::KEY_CHECKOUT_STEPS, array_merge($steps, [$paymentStepCheckout]));

        return $this;
    }

    public function getCheckoutSteps(bool $clear = false): array
    {
        $steps = $this->getData(self::KEY_CHECKOUT_STEPS, $clear);

        if (null === $steps) {
            return [];
        }

        if (!is_array($steps)) {
            return  [];
        }

        return 0 < count($steps) ? $steps : [];
    }

    public function setImpressionCatalogProducts($product): Session
    {
        $items = $this->getImpressionCatalogProducts();

        $this->storage->setData(self::KEY_IMPRESSION_CATALOG_PRODUCTS, array_merge($items, [$product]));

        return $this;
    }

    public function getImpressionCatalogProducts(bool $clear = false): array
    {
        $items = $this->getData(self::KEY_IMPRESSION_CATALOG_PRODUCTS, $clear);

        if (null === $items) {
            return [];
        }

        if (!is_array($items)) {
            return  [];
        }

        return 0 < count($items) ? $items : [];
    }

    public function setImpressionSearchProducts($product): Session
    {
        $items = $this->getImpressionSearchProducts();

        $this->storage->setData(self::KEY_IMPRESSION_SEARCH_PRODUCTS, array_merge($items, [$product]));

        return $this;
    }

    public function getImpressionSearchProducts(bool $clear = false): array
    {
        $items = $this->getData(self::KEY_IMPRESSION_SEARCH_PRODUCTS, $clear);

        if (null === $items) {
            return [];
        }

        if (!is_array($items)) {
            return  [];
        }

        return 0 < count($items) ? $items : [];
    }

    public function setClickImpressionProducts($product): Session
    {
        $items = $this->getClickImpressionProducts();

        $this->storage->setData(self::KEY_CLICK_IMPRESSION_PRODUCTS, array_merge($items, [$product]));

        return $this;
    }

    public function getClickImpressionProducts(bool $clear = false): array
    {
        $items = $this->getData(self::KEY_CLICK_IMPRESSION_PRODUCTS, $clear);

        if (null === $items) {
            return [];
        }

        if (!is_array($items)) {
            return  [];
        }

        return 0 < count($items) ? $items : [];
    }

    public function setProductAdditionalInfo($productInfo): Session
    {
        $items = $this->getProductCategoryPosition();

        $this->storage->setData(self::KEY_PRODUCT_ADDITIONAL_INFO, $productInfo);

        return $this;
    }

    public function getProductAdditionalInfo(bool $clear = false): array
    {
        $items = $this->getData(self::KEY_PRODUCT_ADDITIONAL_INFO, $clear);

        if (null === $items) {
            return [];
        }

        if (!is_array($items)) {
            return  [];
        }

        return 0 < count($items) ? $items : [];
    }

    public function setBehavioralEvent($behavioralEvent): Session
    {
        $events = $this->getBehavioralEvent();

        $this->storage->setData(self::KEY_BEHAVIORAL_EVENT, array_merge($events, [$behavioralEvent]));

        return $this;
    }

    public function getBehavioralEvent(bool $clear = false): array
    {
        $events = $this->getData(self::KEY_BEHAVIORAL_EVENT, $clear);

        if (null === $events) {
            return [];
        }

        if (!is_array($events)) {
            return  [];
        }

        return 0 < count($events) ? $events : [];
    }
}
