<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Plugin;

use Flat101\EnhancedEcommerce\Model\DataLayer\Generator\CheckoutStep;
use Flat101\EnhancedEcommerce\Model\Session;
use Magento\Checkout\Model\PaymentInformationManagement;
use Magento\Sales\Api\OrderRepositoryInterface;

class PaymentInformation
{
    private const PAYMENT_STEP = 3;
    private const STEP_LABEL = 'step 3 - pago';

    /** @var OrderRepositoryInterface */
    private $orderRepository;

    /** @var CheckoutStep */
    private $checkoutStep;

    /** @var Session */
    private $session;

    public function __construct(
        OrderRepositoryInterface $orderRepository,
        CheckoutStep $checkoutStep,
        Session $session
    ) {
        $this->orderRepository = $orderRepository;
        $this->checkoutStep = $checkoutStep;
        $this->session = $session;
    }

    public function afterSavePaymentInformationAndPlaceOrder(PaymentInformationManagement $subject, int $result): int
    {
        $order = $this->orderRepository->get($result);
        $payment = $order->getPayment();

        if (null === $payment) {
            return $result;
        }

        $information = $payment->getAdditionalInformation();

        if (0 === \count($information) || !isset($information['method_title'])) {
            return $result;
        }

        $this->session->setCheckoutStep(
            $this->checkoutStep->onCheckoutStep(self::PAYMENT_STEP, $information['method_title'], self::STEP_LABEL, true)
        );

        return $result;
    }
}
