<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Plugin;

use Flat101\EnhancedEcommerce\Model\DataLayer\Generator\CheckoutStep;
use Flat101\EnhancedEcommerce\Model\Session AS CustomSession;
use Magento\Checkout\Api\Data\ShippingInformationInterface;
use Magento\Checkout\Model\ShippingInformationManagement;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Customer\Model\Session AS CustomerModelSession;

class ShippingInformation
{
    private const REGISTER_STEP = 1;
    private const REGISTER_STEP_LABEL = 'step 1 - registro';

    private const SHIPPING_STEP = 2;
    private const SHIPPING_STEP_LABEL = 'step 2 - entrega';

    /** @var CartRepositoryInterface */
    private $quoteRepository;

    /** @var CustomSession */
    private $customSession;

    /** @var CheckoutStep */
    private $checkoutStep;

    /** @var CustomerModelSession */
    private $customerSession;

    public function __construct(
        CartRepositoryInterface $quoteRepository,
        CustomSession $customSession,
        CheckoutStep $checkoutStep,
        CustomerModelSession $session
    ) {
        $this->quoteRepository = $quoteRepository;
        $this->customSession = $customSession;
        $this->checkoutStep = $checkoutStep;
        $this->customerSession = $session;
    }

    public function afterSaveAddressInformation(
        ShippingInformationManagement $object,
        $result,
        $cartId,
        ShippingInformationInterface $addressInformation
    ) {
        /** @var \Magento\Quote\Model\Quote $quote */
        $quote = $this->quoteRepository->getActive($cartId);

        $registerOption = '';
        if ($this->customerSession->isLoggedIn()) {
            $registerOption = 'Login';
        } else {
            $registerOption = 'Guest';
        }

        $this->customSession->setCheckoutStep(
            $this->checkoutStep->onCheckoutStep(self::REGISTER_STEP, $registerOption, self::REGISTER_STEP_LABEL)
        );

        $this->customSession->setCheckoutStep(
            $this->checkoutStep->onCheckoutStep(self::SHIPPING_STEP, $quote->getShippingAddress()->getShippingDescription(), self::SHIPPING_STEP_LABEL)
        );

        return $result;
    }
}
