# Enhanced Ecommerce M2 Module by Flat 101

## Starting 🚀

Enhanced Ecommerce is a module that tries to solve the problems that we find in each project 
when implementing Enhanced Ecommerce. This solution is an extensible version that, with the 
help and experience of Flat 101's Analytics Department, has been implemented based on the trend 
of the majority of customers.

## Installation

Use the package manager Composer to install module from the Flat 101's private package repository.

First, you need to add the Flat 101's private package repository to the project. Once the 
repository has been added:  

```bash
composer require flat101/enhanced-ecommerce
```
Note: This module has Flat101_Analytics dependence.

```bash
bin/magento setup:upgrade
bin/magento setup:di:compile
bin/magento cache:clean
```

## Configuration

Go to Admin Panel > Flat 101 > Analytics > Enhanced Ecommerce

From here, we can choose different settings.

> Product Identifier

Select between Product Id or SKU.

> Enable Brand

Use Datalayer's brand field.

> Brand Attribute

Select from attribute list which one represents brand attribute.

> Category format

Choose between full breadcrumb path or only main category.

> Category separator

Set character to separate category names.

> Variant use

Variant is a Datalayer's field name. By default, it's used for product variants. 
Flat 101 Analytics Department ask for that this field could be used to set product 
discount percents. So, we can choose between normal use, discount use or none. 

## Data Layers

For the Datalayers listed below, nothing needs to be implemented or modified. Only 
with the installation and configuration of the module the Datalayers will appear in 
the appropriate places.

> #### PRODUCT IMPRESSIONS

It has in mind pagination for position field.

```json
{
  "event": "productList",  
  "ecommerce": {
    "currencyCoce": "EUR",
    "impressions": [
      {
        "id": "WT09",
        "name": "Breathe-Easy Tank",
        "price": "34.0000",
        "category": "Bras & Tanks",
        "brand": "Breathe-Easy Tank",
        "variant": "13%",
        "position": 1
      },
      {
        "id": "WT09",
        "name": "Breathe-Easy",
        "price": "31.0000",
        "category": "Bras & Tanks",
        "brand": "Breathe-Easy Tank",
        "variant": "8%",
        "position": 2
      }
    ]
  }
}
```

> #### PRODUCT CLICK

```json
{
  "event": "productClick",
  "ecommerce": {
    "click": {
      "actionField": {
        "list": "Bras & Tanks"         // Category name
      }
    },
    "products": [
      {
        "id": "WT09",
        "name": "Breathe-Easy Tank",
        "price": "34.0000",
        "category": "Bras & Tanks",    // This data is dragged from listing
        "brand": "Breathe-Easy Tank",
        "variant": "15%",
        "position": 1                  // This data is dragged from listing
      }
    ]
  }
}
```

> #### PRODUCT DETAILS

```json
{
  "event": "productDetail",  
  "ecommerce": {
    "detail": {
      "actionField": {
        "list": "Bras"                 // Category name
      }
    },
    "products": [
      {
        "id": "WT09",
        "name": "Breathe-Easy Tank",
        "price": "34.0000",
        "category": "Bras & Tanks",    // This data is dragged from listing
        "brand": "Breathe-Easy Tank",
        "variant": "15%",
        "position": 1                  // This data is dragged from listing
      }
    ]
  }
}
```

> #### RELATED PRODUCTS

```json
{
  "event": "productList",
  "ecommerce": {
    "currencyCoce": "EUR",
    "impressions": [
      {
        "id": "WT09",
        "name": "Breathe-Easy Tank",
        "price": "34.0000",
        "category": "Bras & Tanks",
        "brand": "Breathe-Easy Tank",
        "variant": "13%",
        "position": 1, 
        "list": "CrossSelling - Related Products"
      },
      {
        "id": "WT09",
        "name": "Breathe-Easy",
        "price": "31.0000",
        "category": "Bras & Tanks",
        "brand": "Breathe-Easy Tank",
        "variant": "8%",
        "position": 2, 
        "list": "CrossSelling - Related Products"
      }
    ]
  }
}
```

> #### ADD TO CART

```json
{
  "event": "addToCart",
  "ecommerce": {
    "currencyCode": "EUR",
    "add": {
      "products": [
        {
          "id": "WT09",
          "name": "Breathe-Easy Tank",
          "price": "34.0000",
          "quantity": 1,
          "category": "Bras & Tanks",    // This data is dragged from listing
          "brand": "Breathe-Easy Tank",
          "variant": "15%",
          "position": 1                  // This data is dragged from listing
        }
      ]
    }
  }
}
```

> #### REMOVE FROM CART

```json
{
  "event": "removeFromCart",
  "ecommerce": {
    "currencyCode": "EUR",
    "remove": {
      "products": [
        {
          "id": "WT11",
          "name": "Breathe-Easy Tank",
          "price": "69.0000",
          "quantity": 1,
          "category": "Bras & Tanks",    // This data is dragged from listing
          "brand": "Breathe-Easy Tank",
          "variant": "15%",
          "position": 1                  // This data is dragged from listing
        }
      ]
    }
  }
}
```

> #### CHECKOUT STEP 1

With current structure of Magento 2 Checkout, this Datalayer is launched when Magento 2 
Checkout Step 2 is loaded. 

```json
{
  "event": "checkoutStep",
  "eventCategory": "enhanced ecommerce",
  "eventAction": "checkout",
  "eventLabel": "step 1 - registro",
  "ecommerce": {
    "currencyCode": "EUR",
    "checkout": {
      "actionField": {
        "step": 1,
        "option": "Guest"                // Guest or Login.
      },
      "products": [
        {
          "id": "WT09",
          "name": "Breathe-Easy Tank",
          "price": "34.0000", 
          "quantity": 1,
          "category": "Bras & Tanks",    // This data is dragged from listing
          "brand": "Breathe-Easy Tank",
          "variant": "15%",
          "position": 1                  // This data is dragged from listing
        }
      ]
    }
  }
}
```

> #### CHECKOUT STEP 2

With current structure of Magento 2 Checkout, this Datalayer is launched when Magento 2
Checkout Step 2 is loaded.

```json
{
  "event": "checkoutStep",
  "eventCategory": "enhanced ecommerce",
  "eventAction": "checkout",
  "eventLabel": "step 2 - entrega",
  "ecommerce": {
    "currencyCode": "EUR",
    "checkout": {
      "actionField": {
        "step": 2,
        "option": "Recogida en Tienda"   // This data is dragged from listing
      },
      "products": [
        {
          "id": "WT09",
          "name": "Breathe-Easy Tank",
          "price": "34.0000",
          "quantity": 1,
          "category": "Bras & Tanks",    // This data is dragged from listing
          "brand": "Breathe-Easy Tank",
          "variant": "15%",
          "position": 1                  // This data is dragged from listing
        }
      ]
    }
  }
}
```

> #### CHECKOUT STEP 3

Appears when success page is loaded.

```json
{
  "event": "checkoutStep",
  "eventCategory": "enhanced ecommerce",
  "eventAction": "checkout",
  "eventLabel": "step 3 - pago",
  "ecommerce": {
    "currencyCode": "EUR",
    "checkout": {
      "actionField": {
        "step": 3,
        "option": "Tarjeta de crédito"
      },
      "products": [
          {
              "id": "WT09",
              "name": "Breathe-Easy Tank",
              "price": "34.0000",
              "quantity": 1,
              "category": "Bras & Tanks",    // This data is dragged from listing
              "brand": "Breathe-Easy Tank",
              "variant": "15%",
              "position": 1                  // This data is dragged from listing
          }
      ]
    }
  }
}
```

> #### PURCHASES

```json
{
  "event": "transactionComplete",
  "ecommerce": {
    "currencyCode": "EUR",
    "purchase": {
      "actionField": {
        "id": "000000033",
        "revenue": "82.000",
        "tax": "3.0100",
        "shipping": "5.0000",
        "coupon": ""
      },
      "products": [
        {
          "id": "WT06",
          "name": "Breathe-Easy Tank",
          "price": "77.0000",
          "quantity": "1.0000",
          "category": "Bras & Tanks",    // This data is dragged from listing
          "brand": "Breathe-Easy Tank",
          "position": 1                  // This data is dragged from listing
        }
      ]
    }
  }
}

```

## Development notes

For Behaviral Events, we need to implement this changes. This events are customized 
and focused in the current project, so, could be necessary check each event separately.

> #### NEWSLETTER EVENT

The event is collected from the newsletter form in the footer and registration form. 
It is necessary that we add in the forms a hidden field named 'locate' where we add 
the origin. By default, the origin can take the values Home, PLP, PDP, Cart, Checkout 
and My Account.

```json
{
   "event": "Newsletter",
   "eventCategory": "Home",
   "eventAction": "Newsletter",
   "eventLabel": "Footer"
}
```

> #### CUSTOMER CREATE ACCOUNT EVENT

The event is launch when event 'customer_register_success' is fired. Additionally, we set custom
cookie in banner when user clicks over. To add this cookie, exists a JS in this module for this
propose. You can use it from code, CMS Block or CMS Page with this snippet:

```html
data-mage-init='{"Flat101_EnhancedEcommerce/js/behavioral_events": {"name": "create_account", "eventCat": "Home"}}'
```

where 'name' is used to cookie name and 'eventCat' is the value.

```json
{
    "event": "Club Vivo",
    "eventCategory": "Home", // Variable: page where user starts proccess (Home or PLP).
    "eventAction": "Registro Club Vivo"
}
```

NOTE: Remind that this codes added into CMS Blocks and Pages could be dangerous when you edit from Admin Panel.

> #### PLP: GRID SORT ORDER

The event is launch when user changes sort order select in toolbar. Any developer changes needed.

```json
{
    "event": "Sort Order",
    "eventCategory": "Sort Order",
    "eventAction": "Posición" // Variable: selected option text
}
```

> #### MENU

The event is launch when user clicks in some menu link. Ensure href links contains next CSS classes:

```html
Main Categories (level 0) => 'menu-top-level-link'
Sub Categories (level 1) => 'menu-middle-level-link'
Sub Sub Categories (level 2) => 'menu-last-level-link'
```

If you prefer, you can change this classes in file:

```html
app/code/Flat101/EnhancedEcommerce/view/frontend/web/js/menu-mixin.js
```

```json
{
    "event": "Menú",
    "eventCategory": "Menú",
    "eventAction": "Subcategory 2",
    "eventLabel": "https://farmaciasvivo.com/belleza/cabello"
}
```

> #### FILTER (BASED ON AMASTY SHOPBY MODULE)

The event is launch when user clicks in some filter link. Ensure href links contains CSS class

```html
am-filter-item-XXXXXXXX
```

If you prefer, you can change this class in file:

```html
app/code/Flat101/EnhancedEcommerce/view/frontend/web/js/amshopby-mixin.js
```

```json
{
   "event": "Filter",
   "eventCategory": "Filter",
   "eventAction": "Marca",
   "eventLabel": "Vichy" 
}
```

## Usage

This module starts to through Datalayers once it is installed. You don't need to 
configure nothing, but you need to check that all the features are in the right mode.

## Contributions
Pull requests are welcome. For major changes, please open an issue first to discuss what you would like to change.


## Authors

* **Juan Antonio Licer** - Flat 101 Team
